#!/usr/bin/python

"""
ac_prediction 1.1
-----------------

The program was written by Oleg Sobolev.
Author will appreciate any questions, comments and 
bug reports to: sobolev@impb.psn.ru

PURPOSE: 
Ac_prediction makes prediction which side chains should be 
checked first with electron density maps because they are 
primary candidates to have alternative conformations.
The prediction is based on atomic shifts during unrestrained 
refinement. (3 macro-cycles of phenix.refine or 10 cycles of
Refmac).

REQUIREMENTS:
-------------
This program requires Python 2.6 or newer (not Python 3.x).
If you have python installed on your computer you can use it.
Otherwise you can try python from PHENIX package
(command: phenix.python).
See below some tips about installation on different operating
systems.

USAGE:
------
Mandatory arguments: two .pdb files.

Optional keys:
-o<output file name>	-Output filename
-r<Rfactor>		-Rwork of the model before unrestrained refinement
-s<resolution>		-Resolution of X-ray data
--refmac		-The refinement was run by Refmac
-t			-Use threshold-based criterion (Default)
-l			-Use likelyhood-based criterion
-n			-Sort output list by residue number

Example (with local python installation)
python ac_prediction.py 1hg7_1.pdb 1hg7_2.pdb
python ac_prediction.py 1hg7_1.pdb 1hg7_2.pdb --refmac -o1hg7.log -r0.12 -s1.15

The program will produce 1 text file file with results.


Also it is possible to use file with parameters:
python ac_prediction.py params.txt

===================params.txt begins========================
[main]
pdb1=1hg7_1.pdb
pdb2=1hg7_2.pdb
log_name=1hg7.log
refmac_refinement=False
threshold=True
resolution=1.15
rfactor=0.12
sort_residue_number=False
===================params.txt ends==========================

REFERENCE:
---------

1. O. V. Sobolev and V. Y. Lunin "Detection of alternative 
conformations by unrestrained refinement". Acta Cryst. D68.1118-1127.
2. O.V. Sobolev. DETAC: tools to detect alternative conformations 
by unrestrained refinement. (2012). 
Computational Crystallography Newsletter. 3.

REMARKS:
--------

======================Installation==========================

WINDOWS:
You can download windows installers of the necessary programs.
Python:		http://python.org/download/

To find out what version of Python you have installed type
"python" in command promt (cmd):

c:\>python
Python 2.6.6 (r266:84297, Aug 24 2010, 18:46:32) [MSC v.1500 32 bit (Intel)] on
win32
Type "help", "copyright", "credits" or "license" for more information.
>>>

If you have just installed python and version number in cmd window 
does not match version you have installed, examine system variable
PATH and make sure that path to your python installation is
situated in the beginning of it.

LINUX:
All necessary programs are packaged for almost every major 
Linux distribution. So if you are on Linux, your package manager 
will probably provide Python.

To find out what version of Python you have installed 
type "python":

oleg@ant:~$ python
Python 2.7.3rc2 (default, Apr 22 2012, 22:30:17)
[GCC 4.6.3] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>>

============================================================
"""

import sys
import math
import ast
from optparse import OptionParser
import ConfigParser


def GetDBtext(resolution, refprogram):
    if refprogram == "phenix":
        if resolution <1.1:
            return \
"""
mean_dr   (prediction mode)
[0.050899, 0.099349999999999994, 0.14780099999999999]
mi 0.010055
si 0.0288002
mo 1000.0
so 1000.0
mi 0.014813
si 0.031441
mo 1000.0
so 1000.0
mean_shift_lik   (prediction mode)
[0.050899, 0.099349999999999994, 0.14780099999999999]
{'m': [0.0, 0.0022000000000000001, 0.014962500000000002, 0.027725000000000003, 0.040487500000000003, 0.053250000000000006, 0.066012500000000002, 0.078774999999999998, 0.091537500000000008, 0.68730009999999997], 'm': [0.0, 0.0014, 0.0056750000000000004, 0.0099500000000000005, 0.014225, 0.018499999999999999, 0.022775, 0.027049999999999998, 0.031324999999999999, 0.22880010000000001]}
{'s': [0.042424242424242427, 0.29696969696969699, 0.10909090909090909, 0.11515151515151516, 0.084848484848484854, 0.090909090909090912, 0.012121212121212121, 0.036363636363636362, 0.21212121212121213], 'm': [0.065476190476190479, 0.40476190476190477, 0.18452380952380953, 0.14285714285714285, 0.053571428571428568, 0.023809523809523808, 0.041666666666666664, 0.011904761904761904, 0.071428571428571425]}
{'s': [0.071545603495357726, 0.5898416166029492, 0.14363735663571819, 0.068814855270344075, 0.03386127799016931, 0.021845985800109231, 0.012015292190060076, 0.0087383943200436912, 0.049699617695248499], 'm': [0.053308823529411763, 0.3483455882352941, 0.27068014705882354, 0.13419117647058823, 0.064797794117647065, 0.03125, 0.017922794117647058, 0.015165441176470588, 0.064338235294117641]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [0.0, 0.0033, 0.020125000000000001, 0.036949999999999997, 0.053774999999999996, 0.070599999999999996, 0.087425000000000003, 0.10425, 0.12107499999999999, 2.0409001], 'm': [0.0, 0.0023999999999999998, 0.0082375, 0.014074999999999999, 0.0199125, 0.025749999999999999, 0.031587499999999998, 0.037425, 0.043262499999999995, 0.80780009999999991]}
{'s': [0.011424794636955907, 0.20791237843451987, 0.16070248324048719, 0.12680577849117175, 0.092720234161080162, 0.070909262581437066, 0.058257010669436317, 0.046643376451704278, 0.22462468133320743], 'm': [0.022591722392915237, 0.20495210554852702, 0.18904753298391469, 0.13175492499548166, 0.084402674859931318, 0.050786191939273452, 0.046448581239833725, 0.031266943791794684, 0.23874932224832821]}
{'s': [0.061972291322426598, 0.59867811813411287, 0.17460461569189983, 0.063061754398707176, 0.032248107057904964, 0.01942875819367022, 0.01209304014671436, 0.008298077097670364, 0.029615237956893577], 'm': [0.06405023058706219, 0.37696310607004863, 0.25074784993144711, 0.12791349869126262, 0.063395861897045991, 0.036535585192571357, 0.020643774149320702, 0.013648261248909386, 0.046101832232332045]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
"""
        else:
            return \
"""
mean_dr   (prediction mode)
[0.096099000000000004, 0.1278, 0.159501]
mi 0.0196385
si 0.037936
mo 1000.0
so 1000.0
mi 0.028373625
si 0.044405
mo 1000.0
so 1000.0
mean_shift_lik   (prediction mode)
[0.096099000000000004, 0.1278, 0.159501]
{'s': [0.0, 0.0051999999999999998, 0.0263375, 0.047475000000000003, 0.068612499999999993, 0.089749999999999996, 0.1108875, 0.132025, 0.15316250000000001, 2.5414000999999997], 'm': [0.0, 0.0037000000000000002, 0.010862500000000001, 0.018024999999999999, 0.025187500000000002, 0.032349999999999997, 0.039512499999999999, 0.046675000000000001, 0.053837499999999996, 1.1103001000000001]}
{'s': [0.0097597597597597601, 0.15870870870870871, 0.15885885885885886, 0.11786786786786786, 0.095795795795795793, 0.073723723723723725, 0.06231231231231231, 0.048798798798798795, 0.27417417417417417], 'm': [0.044546850998463901, 0.21697388632872505, 0.18778801843317972, 0.13940092165898618, 0.08218125960061444, 0.061443932411674347, 0.033410138248847927, 0.027265745007680493, 0.20698924731182797]}
{'s': [0.053922959828684953, 0.56435924559457162, 0.18569880543873682, 0.068564720452024053, 0.036726953739776566, 0.022343197708919217, 0.015441574860032508, 0.011197399313707784, 0.041745143063546535], 'm': [0.053978166135046496, 0.3090098387169235, 0.2542230109169325, 0.14859158093355496, 0.085010557527292335, 0.047452715755424771, 0.026854306123365829, 0.01712790332000539, 0.057751920571454243]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [0.0, 0.0071000000000000004, 0.037475000000000001, 0.067849999999999994, 0.098224999999999993, 0.12859999999999999, 0.15897499999999998, 0.18934999999999999, 0.219725, 2.3926000999999997], 'm': [0.0, 0.0047000000000000002, 0.016324999999999999, 0.027949999999999999, 0.039575000000000006, 0.051200000000000002, 0.062824999999999992, 0.074450000000000002, 0.086074999999999999, 1.5668001]}
{'s': [0.010427528675703858, 0.21567605144247479, 0.1640597844977407, 0.12374000695168579, 0.091762252346193951, 0.071428571428571425, 0.049009384775808136, 0.051963851233924226, 0.22193256864789712], 'm': [0.01800720288115246, 0.187875150060024, 0.18637454981992796, 0.12004801920768307, 0.095438175270108047, 0.062424969987995196, 0.041416566626650657, 0.045318127250900363, 0.24309723889555823]}
{'s': [0.058999708447083146, 0.60014842693948955, 0.15926740703437675, 0.06276339155556733, 0.033581595059503297, 0.021733944710965042, 0.015266770918921785, 0.0111320204617138, 0.037106734872379338], 'm': [0.053825091853689361, 0.41880601904940207, 0.25288432285694229, 0.11623879619011959, 0.054035711778334231, 0.028691114179401372, 0.017458051531674897, 0.012379771126348553, 0.045681121434087667]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
"""
    else:
        if resolution <1.1:
            return \
"""
mean_dr   (prediction mode)
[0.091599, 0.1192, 0.14680100000000001]
mi 0.005353375
si 0.0060145
mo 1000.0
so 1000.0
mi 0.0062895
si 0.0061283
mo 1000.0
so 1000.0
mean_shift_lik   (prediction mode)
[0.091599, 0.1192, 0.14680100000000001]
{'s': [0.0, 0.0011333333333333334, 0.0096316666666666668, 0.01813, 0.026628333333333334, 0.035126666666666667, 0.043624999999999997, 0.052123333333333334, 0.060621666666666671, 1.2029001000000001], 'm': [0.0, 0.00095000000000000011, 0.0024875000000000001, 0.0040250000000000008, 0.0055625000000000006, 0.0071000000000000013, 0.0086375000000000011, 0.010175, 0.011712500000000001, 1.0706751000000001]}
{'s': [0.0019646365422396855, 0.20039292730844793, 0.21611001964636542, 0.14341846758349705, 0.096267190569744601, 0.049115913555992138, 0.049115913555992138, 0.03732809430255403, 0.206286836935167], 'm': [0.006369426751592357, 0.08598726114649681, 0.08598726114649681, 0.098726114649681534, 0.10191082802547771, 0.079617834394904455, 0.060509554140127389, 0.044585987261146494, 0.43630573248407645]}
{'s': [0.058504073068378175, 0.76820538138731176, 0.076524314983954581, 0.02517896815601086, 0.013083189335966429, 0.0091335472722784503, 0.0069118736114539619, 0.0051839052085904712, 0.037274746976055297], 'm': [0.053443406474038668, 0.38431457744948944, 0.2800347599391701, 0.14208125135781013, 0.056050401911796652, 0.028025200955898326, 0.014121225287855746, 0.0093417336519661092, 0.0325874429719748]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [0.0, 0.0014, 0.017364999999999995, 0.033329999999999992, 0.049294999999999992, 0.065259999999999985, 0.081224999999999978, 0.097189999999999985, 0.11315499999999998, 1.2256601], 'm': [0.0, 0.0012000000000000001, 0.003953125, 0.0067062500000000004, 0.009459374999999999, 0.012212499999999999, 0.014965625, 0.017718749999999998, 0.020471875, 1.2579001000000001]}
{'s': [0.0055350553505535052, 0.39360393603936039, 0.21279212792127922, 0.088560885608856083, 0.051045510455104554, 0.030750307503075031, 0.035055350553505532, 0.022140221402214021, 0.16051660516605165], 'm': [0.0022935779816513763, 0.099770642201834861, 0.18807339449541285, 0.15825688073394495, 0.094036697247706427, 0.084862385321100922, 0.063073394495412841, 0.04931192660550459, 0.26032110091743121]}
{'s': [0.068435754189944131, 0.80155959031657353, 0.054236499068901302, 0.015479515828677839, 0.0088454376163873364, 0.0048882681564245811, 0.0064013035381750467, 0.0047718808193668531, 0.035381750465549346], 'm': [0.054598276898417154, 0.66018833901021834, 0.14976958525345621, 0.043979162492486477, 0.020236425566018835, 0.015527950310559006, 0.011320376678020437, 0.0064115407733921059, 0.037968343017431377]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
"""
        
        else:
            return \
"""
mean_dr   (prediction mode)
[0.099499000000000004, 0.12465, 0.14980099999999999]
mi 0.005205875
si 0.0151545
mo 1000.0
so 1000.0
mi 0.0105245
si 0.0201866
mo 1000.0
so 1000.0
mean_shift_lik   (prediction mode)
[0.099499000000000004, 0.12465, 0.14980099999999999]
{'s': [0.0, 0.0020333333333333336, 0.011611666666666666, 0.021190000000000001, 0.030768333333333332, 0.040346666666666663, 0.049924999999999997, 0.059503333333333325, 0.069081666666666666, 1.0103001], 'm': [0.0, 0.0017500000000000003, 0.0036437500000000003, 0.0055374999999999999, 0.0074312500000000004, 0.009325, 0.01121875, 0.013112499999999999, 0.015006249999999999, 0.89192509999999992]}
{'s': [0.0033898305084745762, 0.14237288135593221, 0.15932203389830507, 0.16101694915254236, 0.088135593220338981, 0.071186440677966104, 0.057627118644067797, 0.050847457627118647, 0.26610169491525426], 'm': [0.035335689045936397, 0.088339222614840993, 0.1166077738515901, 0.12014134275618374, 0.11307420494699646, 0.067137809187279157, 0.063604240282685506, 0.045936395759717315, 0.34982332155477031]}
{'s': [0.054121347136679335, 0.71117808937229721, 0.10667016118464159, 0.042065260123181757, 0.018346219368365876, 0.01192504258943782, 0.0069453544751670818, 0.0072074433232865946, 0.041541082426942731], 'm': [0.050616856912256425, 0.34832237979937736, 0.27349244782658827, 0.13778392713017409, 0.069295514816095929, 0.033552404012452436, 0.023636573273377148, 0.013028940389715207, 0.050270955839963105]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [0.0, 0.0024666666666666669, 0.01637708333333333, 0.030287499999999995, 0.044197916666666663, 0.058108333333333324, 0.072018749999999992, 0.085929166666666668, 0.099839583333333329, 2.4078751], 'm': [0.0, 0.002075, 0.0050875, 0.0080999999999999996, 0.011112500000000001, 0.014125, 0.0171375, 0.020150000000000001, 0.023162499999999999, 0.98315010000000003]}
{'s': [0.0084075173095944609, 0.18892185954500496, 0.20870425321463898, 0.14638971315529178, 0.095944609297725025, 0.06577645895153314, 0.044510385756676561, 0.042532146389713157, 0.19881305637982197], 'm': [0.01423785594639866, 0.12562814070351758, 0.17169179229480738, 0.11557788944723618, 0.099664991624790616, 0.071189279731993294, 0.056951423785594639, 0.040201005025125629, 0.30485762144053602]}
{'s': [0.054734508603399137, 0.7314472711918083, 0.096115274992082764, 0.032249551356486855, 0.017892958935923148, 0.011084133854111685, 0.0076533305183152118, 0.0063865723635595905, 0.042436398184313315], 'm': [0.052469702533969889, 0.44610723466764596, 0.23549394050679398, 0.11081527726771943, 0.055499449136981273, 0.02859897172236504, 0.016984943077488063, 0.010145060594932059, 0.043885420492104296]}
{'s': [], 'm': []}
{'s': [], 'm': []}
{'s': [], 'm': []}
"""


def Get_acc_char(res):
    return 'o' if res.acc >= 50 else 'i'

def Crit_mean_dr(res,db):
    #prediction = {'m':False, 's':False}
    acc_ch=Get_acc_char(res)
    for chain in ['m','s']:
        mdr = res.get_mean_dr(chain)
        #print mdr
        if mdr > db[chain+acc_ch]:
            res.prediction[chain] = True 
        #res.str_pred[chain] = "mean_dr: %.4f; threshold: %.4f; mean_dr-thr: %.4f" % (mdr, db[chain+acc_ch], mdr-db[chain+acc_ch])
        res.str_pred[chain]['str'] = "mean_dr/thr: %.4f; mean_dr: %.4f"
        res.str_pred[chain]['vals'] = (mdr/db[chain+acc_ch], mdr)

def Crit_mean_shift_lik(res,db):
    # Use this criterion
    #res.prediction = {'m':False, 's':False}
    acc_ch=Get_acc_char(res)
    for chain in ['m','s']:
        mdr = res.get_mean_dr(chain)
        i = 1
        while i < len(db[acc_ch]['bins'][chain])-1 and db[acc_ch]['bins'][chain][i] < mdr:
            i += 1
        i -= 1
        res.prediction[chain] = db[acc_ch]['alt'][chain][i] > db[acc_ch]['noalt'][chain][i] 
        #res.str_pred[chain] = "ln(Palt/Pnoalt): %.4f; Palt: %.4f; Pnoalt: %.4f" % (math.log(db[acc_ch]['alt'][chain][i]/max(db[acc_ch]['noalt'][chain][i],0.0000000001)),db[acc_ch]['alt'][chain][i], db[acc_ch]['noalt'][chain][i])
        res.str_pred[chain]['str'] = "ln(Palt/Pnoalt): %.4f; Palt: %.4f; Pnoalt: %.4f"
        res.str_pred[chain]['vals'] = (math.log(db[acc_ch]['alt'][chain][i]/max(db[acc_ch]['noalt'][chain][i],0.0000000001)), db[acc_ch]['alt'][chain][i], db[acc_ch]['noalt'][chain][i])  

criteria = {'mean_dr':Crit_mean_dr, 'mean_shift_lik':Crit_mean_shift_lik}

def isresidue(rname):
    return rname in ["ALA","ARG","ASN","ASP","CYS","GLN","GLU","GLY","HIS","ILE","LEU","LYS","MET","MSE","PHE","PRO","SER","THR","TRP","TYR","VAL"]

def ismainchain(aname):
    return aname in ['N','CA','C','O']

def GetResolution(fname):
    f = open(fname,"r")
    res = -1
    for line in f.readlines():
        if line[:48] == "REMARK   3   RESOLUTION RANGE HIGH (ANGSTROMS) :":
            res = float(line[49:])
    f.close()
    return res

def GetRfactor(fname):
    f = open(fname,"r")
    rf = -1
    for line in f.readlines():
        if line[:47] == "REMARK   3   R VALUE            (WORKING SET) :":
            rf = float(line[49:])
    f.close()
    return rf

def ReadDB(crit, res, rf,refprog):
    alldb = GetDBtext(res, refprog)
    lines = alldb.split('\n')
    n=1
    l = lines[n].split()
    while l[0] != crit:
        n += 1
        l = lines[n].split()
    n += 1
    resrange = ast.literal_eval(lines[n])
    rfbin = 0
    for r in resrange[1:-1]:
        if r<rf:
            rfbin +=1
    mult = 6
    thresh_crit = crit == 'mean_dr' or crit == 'onemax_dr'
    if thresh_crit:
        mult = 4
    strtoskip = mult*rfbin
    n += strtoskip
    prdata = {}
    if thresh_crit:
        for i in range(4):
            n += 1
            l = lines[n].split()
            prdata[l[0]] = float(l[1])
    else:
        prdata['i']={}
        prdata['o']={}
        for acc in ['i','o']:
            for subd in ['bins','alt','noalt']:
                n += 1
                l = lines[n]
                prdata[acc][subd]=ast.literal_eval(l)
    return prdata
            
def ReadPDBs(pdb1, pdb2,logger,accs=[]):

    def GoodLine(line):
        return line[:6]=="ATOM  " and line[12:16].strip()[0] != 'H' and isresidue(l[17:20])
    
    def MakeHash(a):
        #Hash contains atom ser number, atom name, resname, chainID,resnumber, inscode
        return "%i%s%s%c%i%c" % (a.sernumber, a.aname, a.resname, a.chainID, a.resseq,a.inscode)
    
    #read atoms from 1st pdb
    atoms1 = {}
    in1 = open(pdb1,'r')
    for l in in1.readlines():
        if GoodLine(l) :
            a = CAtom(l)
            atoms1[MakeHash(a)] = a
    
    logger.PrintL("Atoms read from 1st pdb:\t%i" % len(atoms1))
    match_atoms = 0    
    reslist = []
    in2 = open(pdb2,'r')
    crn = -99999
    cr_ins = ''
    acc_present = len(accs) > 0
    for l in in2.readlines():
        if GoodLine(l):
            a = CAtom(l)
            # everything OK, parsing
            if crn != a.resseq or cr_ins != a.inscode:
                #make new residue
                crn = a.resseq
                cr_ins = a.inscode
                curres = CResidue(a.resname, accs[crn] if acc_present else 0, crn,a.inscode, a.chainID)
                reslist.append(curres)
            h = MakeHash(a)
            if h in atoms1:
                a.calc_dr_to_other(atoms1[h])
                curres.add_atom(a)
                match_atoms +=1
    logger.PrintL("Match atoms from 2nd pdb:\t%i" % match_atoms)
    logger.PrintL("Total residues for analysis:\t %i" % len(reslist))
    for r in reslist:
        r.finalize()
    in1.close()
    in2.close()
    errcode = 0
    if match_atoms == 0:
        errcode = 1
    return reslist, errcode

def PrintAnswer(reslist,logger, order):
    nalt = 0
    logger.PrintL("Candidates for side chain alternative conformation:")
    logger.PrintL("#res\tchain")
    altpred_list = [r for r in reslist if r.prediction['s']]
    altpred_list.sort(key=order['funct'], reverse=order['order'])
    for r in altpred_list:
        if r.prediction['s']:
            nalt+=1
            #ans_s = str(r.resnum)+'\t'+r.str_pred['s']
            ans_s = "%i%c\t%c\t%s" % (r.resnum, r.inscode, r.chain_id, r.str_pred['s']['str'] % r.str_pred['s']['vals'])
            logger.PrintL(ans_s)
    logger.PrintL("Total: %i" % nalt)
            
    """
    logger.PrintL("Candidates for main chain alternative conformation:")
    for r in reslist:
        if r.prediction['m']:
            ans_s = str(r.resnum)+'\t'+r.str_pred['m']
            logger.PrintL(ans_s)
    """


def MakePrediction(reslist, db, critname):
    for r in reslist:
        ans = criteria[critname](r, db)

def run(args):
    parser = OptionParser(usage="\n1: %prog <file with parameters>\n2: %prog [options] inp1.pdb inp2.pdb", version="%prog 1.0")
    parser.add_option("-o", "--out", action="store", type="string", dest="out", default="", help="output name")
    parser.add_option("-l", action="store_true", dest="pred_type_lik", default=False, help="Use likelyhood criterion")
    parser.add_option("-t", action="store_false", dest="pred_type_lik", help="Use threshold criterion")
    parser.add_option("-s", "--resolution", action="store", type="float", dest="resolution", help="data resolution, default=1.15", default=-1)
    parser.add_option("-r", "--rfactor", action="store", type="float", dest="rfactor", help="rfactor before unrestrained refinement, default=0.15", default=-1)
    parser.add_option("-n", "--sort_number", action="store_true", dest="sort_res_num", default=False, help="Output residue list sorted by residue number")
    parser.add_option("--refmac", action="store_true", dest="refmac_refinement", default=False, help="Use this option if refinement was carried out by Refmac")
    #print args
    (options, pos_args) = parser.parse_args(args)
    # parsing and checking command line arguments
    
    # setting defaults to variables
    name_pdb1 = ""
    name_pdb2 = ""
    log_name=""
    critname = 'mean_dr'
    resolution = -1
    rfactor = -1
    sort_r_n = False
    refmac_refinement = False
    
    nargs = len(pos_args)

    if  nargs != 2 and nargs != 1:
        print "Please specify correct arguments"
        parser.print_help()
        return 1
    conf = ConfigParser.RawConfigParser()
    if nargs == 1:
        # provided config file
        try:
            f = open(pos_args[0],'r')
            f.close
        except IOError:
            print "Cannot open file with parameters." 
            return
        conf.read(pos_args[0])
        print "from cfg:"
        if conf.has_section('main'):
            if conf.has_option('main', 'pdb1'):
                name_pdb1 = conf.get('main', 'pdb1')
            if conf.has_option('main', 'pdb2'):
                name_pdb2 = conf.get('main', 'pdb2')
            if conf.has_option('main', 'threshold'):
                if not conf.getboolean('main', 'threshold'):
                    critname = 'mean_shift_lik'
            if conf.has_option('main', 'sort_residue_number'):
                sort_r_n = conf.getboolean('main', 'sort_residue_number')
            if conf.has_option('main', 'resolution'):
                resolution = conf.getfloat('main', 'resolution')
            if conf.has_option('main', 'rfactor'):
                rfactor = conf.getfloat('main', 'rfactor')
            if conf.has_option('main','log_name'):
                log_name = conf.get('main','log_name')
            if conf.has_option('main','refmac_refinement'):
                refprog = conf.getboolean('main','refmac_refinement')
    else:
        name_pdb1 = pos_args[0]
        name_pdb2 = pos_args[1]
        
    list_to_open = [name_pdb1, name_pdb2]

    
    # overlapping with command-line args    
    if options.resolution > 0:
        resolution = options.resolution
    if options.rfactor > 0:
        rfactor = options.rfactor
    if options.pred_type_lik:
        critname = 'mean_shift_lik'
    if options.out != "":
        log_name = options.out 
    if options.sort_res_num:
        sort_r_n = options.sort_res_num
    if options.refmac_refinement:
        refmac_refinement = options.refmac_refinement
        
    if refmac_refinement:
        refprog = "refmac"
    else:
        refprog = "phenix"
    
    # Checking the presence of all necessary files
    for fname in list_to_open:
        try:
            f = open(fname,'r')
            f.close
        except IOError:
            print "Cannot open", fname
            return

    if log_name == "":
        log_name = name_pdb1+'.log'
    logger = CLogger(log_name)
    logger.PrintHeader()

    nwarnings = 0
    if resolution == -1:
        resolution = GetResolution(name_pdb1)
    if rfactor == -1:
        rfactor = GetRfactor(name_pdb1)
    
    if resolution == -1:
        logger.PrintBoth("Warning!!! Can't obtain resolution, assuming 1.15")
        resolution = 1.15
        nwarnings += 1
    if rfactor == -1:
        logger.PrintBoth("Warning!!! Can't obtain Rfactor, assuming 0.13")
        rfactor = 0.13
        nwarnings += 1

    
    #Everything seems ok, reading files
    #First database
    db = ReadDB(critname,resolution, rfactor, refprog)
    #print "db ready"
    #print "Database:",db['i']['bins']
    #print "Database:",db['i']['alt']
    #print "Database:",db['i']['noalt']

    accs = []
    #print "accs ready"
    reslist, errc = ReadPDBs(name_pdb1, name_pdb2,logger, accs)
    logger.PrintL("Command line arguments: "+str(args))
    logger.PrintL("Input pdb 1: "+name_pdb1)
    logger.PrintL("Input pdb 2: "+name_pdb2)
    logger.PrintL("Refinement program: "+refprog)
    logger.PrintL("Resolution : %.4f" % (resolution))
    logger.PrintL("Rfactor    : %.4f" % (rfactor))
    logger.PrintL("Criterion  : "+critname)
    logger.PrintL()
    if errc == 1:
        logger.PrintBoth("Error: No matched atoms were found in pdb models.")
        return 6
    elif errc !=0:
        print "Error during reading PDB files"
        return 4
    if len(reslist) == 0:
        print "Read 0 residues form PDB files"
        return 5
    logger.PrintBoth(str(nwarnings)+" warnings")
    if nwarnings == 0:
        logger.PrintBoth("Everything OK")
    else:
        logger.PrintBoth("There are warnings, the precision of predictions will be significantly lower!")
    logger.PrintL()
    logger.PrintL("Total residues read: "+str(len(reslist)))
    MakePrediction(reslist,db,critname)
    sort_order = {}
    if sort_r_n:
        sort_order['funct'] = lambda residue: residue.resnum
        sort_order['order'] = False
    else:
        sort_order['funct'] = lambda residue: residue.str_pred['s']['vals'][0]
        sort_order['order'] = True 
    PrintAnswer(reslist, logger, sort_order)
    logger.PrintBoth("All done.")
    
class CAtom(object):
    def __init__(self, line):
        """
        7 - 11 Integer serial Atom serial number.
        13 - 16 Atom name Atom name.
        17 Character altLoc Alternate location indicator.
        18 - 20 Residue name resName Residue name.
        22 Character chainID Chain identifier.
        23 - 26 Integer resSeq Residue sequence number.
        27 AChar iCode Code for insertion of residues.
        31 - 38 Real(8.3) x Orthogonal coordinates for X in
        Angstroms.
        39 - 46 Real(8.3) y Orthogonal coordinates for Y in
        Angstroms.
        47 - 54 Real(8.3) z Orthogonal coordinates for Z in
        Angstroms.
        55 - 60 Real(6.2) occupancy Occupancy.
        61 - 66 Real(6.2) tempFactor Temperature factor.
        73 - 76 LString(4) segID Segment identifier, left-justified.
        77 - 78 LString(2) element Element symbol, right-justified.
        79 - 80 LString(2) charge Charge on the atom.
        """
        self.dr = -1
        self.sernumber = int(line[6:11])
        self.aname = line[12:16].strip()
        self.altloc = line[16]
        self.resname = line[17:20]
        self.chainID = line[21]
        self.resseq = int(line[22:26])
        self.inscode = line[26]
        self.x = float(line[30:38])
        self.y = float(line[38:46])
        self.z = float(line[46:54])
        self.occupancy = float(line[54:60])
        self.bfactor = float(line[60:66])
        self.chain = 'm' if ismainchain(self.aname) else 's'
    
    def calc_dr_to_other(self,a):
        self.dr = ((self.x-a.x)*(self.x-a.x)+(self.y-a.y)*(self.y-a.y)+(self.z-a.z)*(self.z-a.z))**0.5 
        

class CLogger(object):
    def __init__(self, fname):
        self.logfile = open(fname,'w')

    def __del__(self):
        self.logfile.close()
        
    def PrintL(self,line=""):
        self.logfile.write(line+'\n')
        
    def PrintBoth(self, line=""):
        self.logfile.write(line+'\n')
        print line
    
    def PrintHeader(self):
        self.PrintBoth(
"""    ac_prediction ver. 1.1 (09.11.2012)

"""
)
        

class CResidue(object):

    def __init__(self, resname, acc, resnum, inscode, chain):
        '''
        Constructor
        atomlist - list of atoms in residue
        resname - name of residue (3 symbol)
        acc - solvent accessibility as in dssp
        '''
        self.atomlist = []
        self.resname = resname
        self.acc = acc
        self.resnum = resnum
        self.inscode = inscode
        self.chain_id = chain
        
        # here must be evaluation of mainalt and sidealt
        self.mainalt = False
        self.sidealt = False
        self.prediction = {'m':False, 's':False}
        self.str_pred = {'m':{'str':"", 'vals':None}, 's':{'str':"", 'vals':None}}
    
    def write(self):
        print self.resname, [x.dr for x in self.atomlist]
        
    def get_atoms_dr(self, chain):
        if chain != 'm' and chain !='s':
            return [x.dr for x in self.atomlist]
        else:
            return [x.dr for x in self.atomlist if x.chain == chain]
        
    def get_alt_atoms_dr(self, chain):
        if chain != 'm' and chain !='s':
            return [x.dr for x in self.atomlist if x.altloc != ' ']
        else:
            return [x.dr for x in self.atomlist if x.chain == chain and x.altloc != ' ']
        
    def add_atom(self, atom):
        self.atomlist.append(atom)
        if atom.chain == 'M' and atom.altloc != '':
            self.mainalt = True
        if atom.chain == 'S' and atom.altloc != '':
            self.sidealt = True
    
    
    def get_max_dr(self,chain):
        max_dr = 0
        chain = chain.upper()
        for atom in self.atomlist:
            if atom.chain.upper() == chain:
                if atom.dr > max_dr:
                    max_dr = atom.dr
        return max_dr

    def get_mean_dr(self,chain):
        mean_dr = 0
        natom = 0
        chain = chain.upper()
        for atom in self.atomlist:
            if atom.chain.upper() == chain:
                mean_dr += atom.dr
                natom += 1
        if natom == 0:
            return 0
        else:
            return mean_dr/natom

    def get_sorted_drs(self,chain):
        atomsdr = []
        chain = chain.upper()
        for atom in self.atomlist:
            if atom.chain.upper() == chain.upper():
                atomsdr.append(atom.dr)
        atomsdr.sort()
        return atomsdr
                
        
    def finalize(self):
        # here must be evaluation of mainalt and sidealt
        for atom in self.atomlist:
            if atom.chain == 'm' and atom.altloc != " ":
                self.mainalt = True
            if atom.chain == 's' and atom.altloc != " ":
                self.sidealt = True


if __name__ == '__main__':
    run(sys.argv[1:])
